package compiler;

import java_cup.runtime.Symbol;

%%

%notunix
%ignorecase
%cup
%char
%line
%state COMMENT

%eofval{
return new Symbol (sym.EOF);
%eofval}

%{
  private int charsByNow=0;
  private int brackets=0;
  private ParseTreeNode lastNode=null;
%}

CRKA=[A-Za-z]
STEVILKA=[0-9]

%%

<YYINITIAL> "and"       {lastNode=new ParseTreeNode (sym.AND, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.AND, lastNode);}
<YYINITIAL> "array"     {lastNode=new ParseTreeNode (sym.ARRAY, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.ARRAY, lastNode);}
<YYINITIAL> "begin"     {lastNode=new ParseTreeNode (sym.BEGIN, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.BEGIN, lastNode);}
<YYINITIAL> "case"      {lastNode=new ParseTreeNode (sym.CASE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.CASE, lastNode);}
<YYINITIAL> "const"     {lastNode=new ParseTreeNode (sym.CONST, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.CONST, lastNode);}
<YYINITIAL> "div"       {lastNode=new ParseTreeNode (sym.DIV, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.DIV, lastNode);}
<YYINITIAL> "do"        {lastNode=new ParseTreeNode (sym.DO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.DO, lastNode);}
<YYINITIAL> "downto"    {lastNode=new ParseTreeNode (sym.DOWNTO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.DOWNTO, lastNode);}
<YYINITIAL> "else"      {lastNode=new ParseTreeNode (sym.ELSE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.ELSE, lastNode);}
<YYINITIAL> "end"       {lastNode=new ParseTreeNode (sym.END, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.END, lastNode);}
<YYINITIAL> "file"      {lastNode=new ParseTreeNode (sym.FILE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.FILE, lastNode);}
<YYINITIAL> "for"       {lastNode=new ParseTreeNode (sym.FOR, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.FOR, lastNode);}
<YYINITIAL> "function"  {lastNode=new ParseTreeNode (sym.FUNCTION, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.FUNCTION, lastNode);}
<YYINITIAL> "goto"      {lastNode=new ParseTreeNode (sym.GOTO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.GOTO, lastNode);}
<YYINITIAL> "if"        {lastNode=new ParseTreeNode (sym.IF, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.IF, lastNode);}
<YYINITIAL> "in"        {lastNode=new ParseTreeNode (sym.IN, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.IN, lastNode);}
<YYINITIAL> "label"     {lastNode=new ParseTreeNode (sym.LABEL, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.LABEL, lastNode);}
<YYINITIAL> "mod"       {lastNode=new ParseTreeNode (sym.MOD, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.MOD, lastNode);}
<YYINITIAL> "nil"       {lastNode=new ParseTreeNode (sym.NIL, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.NIL, lastNode);}
<YYINITIAL> "not"       {lastNode=new ParseTreeNode (sym.NOT, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.NOT, lastNode);}
<YYINITIAL> "of"        {lastNode=new ParseTreeNode (sym.OF, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.OF, lastNode);}
<YYINITIAL> "or"        {lastNode=new ParseTreeNode (sym.OR, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.OR, lastNode);}
<YYINITIAL> "packed"    {lastNode=new ParseTreeNode (sym.PACKED, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PACKED, lastNode);}
<YYINITIAL> "procedure" {lastNode=new ParseTreeNode (sym.PROCEDURE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PROCEDURE, lastNode);}
<YYINITIAL> "program"   {lastNode=new ParseTreeNode (sym.PROGRAM, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PROGRAM, lastNode);}
<YYINITIAL> "record"    {lastNode=new ParseTreeNode (sym.RECORD, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.RECORD, lastNode);}
<YYINITIAL> "repeat"    {lastNode=new ParseTreeNode (sym.REPEAT, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.REPEAT, lastNode);}
<YYINITIAL> "set"       {lastNode=new ParseTreeNode (sym.SET, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.SET, lastNode);}
<YYINITIAL> "then"      {lastNode=new ParseTreeNode (sym.THEN, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.THEN, lastNode);}
<YYINITIAL> "to"        {lastNode=new ParseTreeNode (sym.TO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.TO, lastNode);}
<YYINITIAL> "type"      {lastNode=new ParseTreeNode (sym.TYPE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.TYPE, lastNode);}
<YYINITIAL> "until"     {lastNode=new ParseTreeNode (sym.UNTIL, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.UNTIL, lastNode);}
<YYINITIAL> "var"       {lastNode=new ParseTreeNode (sym.VAR, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.VAR, lastNode);}
<YYINITIAL> "while"     {lastNode=new ParseTreeNode (sym.WHILE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.WHILE, lastNode);}
<YYINITIAL> "with"      {lastNode=new ParseTreeNode (sym.WITH, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.WITH, lastNode);}

<YYINITIAL> ({STEVILKA})+(("."({STEVILKA})+)|("e"("+"|"-")?({STEVILKA})+)|("."({STEVILKA})+)("e"("+"|"-")?({STEVILKA})+)) {lastNode=new ParseTreeNode (sym.ABSOLUTNOSTEVILO, new Float (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.ABSOLUTNOSTEVILO, lastNode);}
<YYINITIAL> {STEVILKA}+                                                                                                   {lastNode=new ParseTreeNode (sym.NARAVNOSTEVILO, new Integer (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.NARAVNOSTEVILO, lastNode);}
<YYINITIAL> {CRKA}({CRKA}|{STEVILKA})*                                                                                    {lastNode=new ParseTreeNode (sym.IME, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.IME, lastNode);}
<YYINITIAL> "'"([^"'"])+"'"                                                                                               {lastNode=new ParseTreeNode (sym.NIZ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.NIZ, lastNode);}

<YYINITIAL> ":=" {lastNode=new ParseTreeNode (sym.PRIREJANJE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PRIREJANJE, lastNode);}
<YYINITIAL> ":"  {lastNode=new ParseTreeNode (sym.DVOPICJE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.DVOPICJE, lastNode);}
<YYINITIAL> ";"  {lastNode=new ParseTreeNode (sym.PODPICJE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PODPICJE, lastNode);}
<YYINITIAL> ","  {lastNode=new ParseTreeNode (sym.VEJICA, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.VEJICA, lastNode);}
<YYINITIAL> ".." {lastNode=new ParseTreeNode (sym.PIKAPIKA, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PIKAPIKA, lastNode);}
<YYINITIAL> "."  {lastNode=new ParseTreeNode (sym.PIKA, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PIKA, lastNode);}
<YYINITIAL> "<>" {lastNode=new ParseTreeNode (sym.NEENACAJ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.NEENACAJ, lastNode);}
<YYINITIAL> ">=" {lastNode=new ParseTreeNode (sym.VECJEENAKO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.VECJEENAKO, lastNode);}
<YYINITIAL> "<=" {lastNode=new ParseTreeNode (sym.MANJSEENAKO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.MANJSEENAKO, lastNode);}
<YYINITIAL> ">"  {lastNode=new ParseTreeNode (sym.VECJE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.VECJE, lastNode);}
<YYINITIAL> "<"  {lastNode=new ParseTreeNode (sym.MANJSE, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.MANJSE, lastNode);}
<YYINITIAL> "="  {lastNode=new ParseTreeNode (sym.ENACAJ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.ENACAJ, lastNode);}
<YYINITIAL> "("  {lastNode=new ParseTreeNode (sym.OKLEPAJ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.OKLEPAJ, lastNode);}
<YYINITIAL> ")"  {lastNode=new ParseTreeNode (sym.ZAKLEPAJ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.ZAKLEPAJ, lastNode);}
<YYINITIAL> "["  {lastNode=new ParseTreeNode (sym.OGLATIOKLEPAJ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.OGLATIOKLEPAJ, lastNode);}
<YYINITIAL> "]"  {lastNode=new ParseTreeNode (sym.OGLATIZAKLEPAJ, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.OGLATIZAKLEPAJ, lastNode);}
<YYINITIAL> "+"  {lastNode=new ParseTreeNode (sym.PLUS, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PLUS, lastNode);}
<YYINITIAL> "-"  {lastNode=new ParseTreeNode (sym.MINUS, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.MINUS, lastNode);}
<YYINITIAL> "*"  {lastNode=new ParseTreeNode (sym.KRAT, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.KRAT, lastNode);}
<YYINITIAL> "/"  {lastNode=new ParseTreeNode (sym.DELJENO, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.DELJENO, lastNode);}
<YYINITIAL> "^"  {lastNode=new ParseTreeNode (sym.PUSCICA, new String (yytext ()), yyline, yychar-charsByNow); return new Symbol (sym.PUSCICA, lastNode);}

<YYINITIAL> [ \t\f\r] { }
<YYINITIAL> \n        {charsByNow=yychar+1;}

<YYINITIAL> "{" {yybegin (COMMENT); brackets++;}
<COMMENT>   "{" {brackets++;}
<COMMENT>   "}" {brackets--; if (brackets<=0) yybegin (YYINITIAL);}
<COMMENT>   ([^"{"^"}"])* {/*System.out.println ("Komentar: "+yytext ());*/ if (lastNode!=null) lastNode.addComment (yytext ());}

<YYINITIAL, COMMENT> . {System.out.println ("Neznan vhod na polozaju ("+yyline+", "+(yychar-charsByNow)+"): "+yytext ());}