package compiler;

import java.util.Vector;

class CTree {

  private CTreeNode root;

  public Vector constants, types, variables, functions;
  public FunctionEntry main;
  public TypeEntry voidType;
  public int loopCount, typeCount, iteCount;

  public CTree (ParseTreeNode source) {
    root=new CTreeNode (source, this);
    constants=new Vector ();
    types=new Vector ();
    functions=new Vector ();
    main=new FunctionEntry (root);
    functions.add (main);
    loopCount=0;
    typeCount=0;
    iteCount=0;
  }

  public ConstantEntry lookupConstant (String name, String prefix) {
    int i;
    ConstantEntry tempCE;
    String tempP=new String (prefix);

    while (tempP.compareTo ("X")!=0) {
      for (i=0; i<constants.size (); i++) {
        tempCE=(ConstantEntry)(constants.elementAt (i));
        if (tempCE.name.compareTo (tempP+name)==0)
          return tempCE;
      }
      if (tempP.length ()>0) {
        tempP=tempP.substring (0, tempP.length ()-1);
        tempP=Auxiliary.getPrefix (tempP);
      } else
        tempP="X";
    }
    return null;
  }

  public TypeEntry lookupType (CTreeNode node, boolean create) {
    int i;
    TypeEntry newType, tempTE;
    String tempP;

    newType=new TypeEntry (node);
    if (node!=null)
      tempP=new String (node.prefix);
    else
      tempP="";
    while (tempP.compareTo ("X")!=0) {
      for (i=0; i<types.size (); i++) {
        tempTE=(TypeEntry)(types.elementAt (i));
        if ((tempTE.name.compareTo (tempP+newType.actualType)==0) && ((newType.arrayPart.length ()==0) || (tempTE.arrayPart.compareTo (newType.arrayPart)==0)))
          return tempTE;
      }
      if (tempP.length ()>0) {
        tempP=tempP.substring (0, tempP.length ()-1);
        tempP=Auxiliary.getPrefix (tempP);
      } else
        tempP="X";
    }
    for (i=0; i<types.size (); i++) {
      tempTE=(TypeEntry)(types.elementAt (i));
      if ((tempTE.name.compareTo ("X")==0) && (tempTE.actualType.compareTo (newType.actualType)==0) && ((newType.arrayPart.length ()==0) || (tempTE.arrayPart.compareTo (newType.arrayPart)==0)))
        return tempTE;
    }
    if (create) {
      types.add (newType);
      if (newType.actualType.compareTo ("void")==0)
        voidType=newType;
    }
    return newType;
  }

  public VariableEntry lookupVariable (String name, FunctionEntry function) {
    Vector names=new Vector ();

    names.add (name);
    return lookupVariable (names, function);
  }

  public VariableEntry lookupVariable (Vector names, FunctionEntry function) {
    int i, j, k, iCount=1, step;
    VariableEntry tempVE, finalVE=null;
    ParameterEntry tempPE;
    String tempI=new String (""), firstName;
    Vector searchedVars;
    CTreeNode record, tempCTN, tempCTN1, tempCTN2;

    searchedVars=function.variables;
    for (i=0; i<names.size (); i++) {
      firstName=(String)(names.elementAt (i));
      for (j=0; j<searchedVars.size (); j++) {
        tempVE=(VariableEntry)(searchedVars.elementAt (j));
        if (tempVE.name.compareTo (firstName)==0)
          finalVE=tempVE;
      }
      if (i==0) {
        if (finalVE==null)
          for (j=0; j<function.parameters.size (); j++) {
            tempPE=(ParameterEntry)(function.parameters.elementAt (j));
            if (tempPE.name.compareTo (firstName)==0)
              finalVE=(new VariableEntry (tempPE.name, tempPE.type));
          }
        if (finalVE==null)
          while (iCount>0) {
            iCount=0;
            tempI+="I";
            for (j=0; j<function.parameters.size (); j++) {
              tempPE=(ParameterEntry)(function.parameters.elementAt (j));
              if (Auxiliary.getPrefix (tempPE.name).length ()>=tempI.length ())
                iCount++;
              if (tempPE.name.compareTo (tempI+firstName)==0)
                finalVE=(new VariableEntry (tempPE.name, tempPE.type));
            }
          }
        if (finalVE==null)
          for (j=0; j<function.rootNode.tree.main.variables.size (); j++) {
            tempVE=(VariableEntry)(function.rootNode.tree.main.variables.elementAt (j));
            if (tempVE.name.compareTo (firstName)==0)
              finalVE=tempVE;
          }
      }
      if (i<names.size ()-1) {
        tempCTN=(CTreeNode)(finalVE.type.node);
        if (tempCTN.name.compareTo ("def_tipa")==0)
          tempCTN=(CTreeNode)(tempCTN.children.elementAt (2));
        if (tempCTN.name.compareTo ("tip")==0) {
          tempCTN1=(CTreeNode)(tempCTN.children.elementAt (0));
          if (tempCTN1.name.compareTo ("struct")!=0) {
            while (tempCTN1.symbolNum!=sym.IME)
              tempCTN1=(CTreeNode)(tempCTN1.children.elementAt (0));
            tempCTN=lookupType (tempCTN1, false).node;
          }
        }
        tempCTN1=(CTreeNode)(((CTreeNode)(tempCTN.children.elementAt (2))).children.elementAt (0));
        searchedVars=new Vector ();
        for (j=0; j<tempCTN1.children.size (); j++) {
          record=(CTreeNode)(tempCTN1.children.elementAt (j));
          step=2;
          if (record.children.size ()>0)
            for (k=1; k<record.children.size(); k+=step) {
              tempCTN2=(CTreeNode)(record.children.elementAt (k));
              if (tempCTN2.varType.type==Constants.TYPE_ARRAY)
                step=3;
              searchedVars.add (new VariableEntry (tempCTN2.name, tempCTN2.varType));
            }
        }
      }
    }
    return finalVE;
  }

  public FunctionEntry lookupFunction (String name, String prefix) {
    int i;
    FunctionEntry tempFE;
    String tempP=new String (prefix);

    while (tempP.compareTo ("X")!=0) {
      for (i=0; i<functions.size (); i++) {
        tempFE=(FunctionEntry)(functions.elementAt (i));
        if (tempFE.name.compareTo (tempP+name)==0)
          return tempFE;
      }
      if (tempP.length ()>0) {
        tempP=tempP.substring (0, tempP.length ()-1);
        tempP=Auxiliary.getPrefix (tempP);
      } else
        tempP="X";
    }
    return null;
  }

  public void removeDeclarations () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.removeDeclarations ();
  }

  public void unLoop () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.unLoop ();
  }

  public void unCase () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.unCase ();
  }

  public void unIf () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.unIf ();
  }

  public void unNestFunctions () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).unNestFunctions ();
    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.unNestFunctions ();
  }

  public void unWith () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.unWith (new Vector ());
  }

  public void names () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.names ();
  }

  public void casts () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.casts ();
  }

  public void byReference () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.byReference ("", -1, Constants.NONE);
  }

  public void finalConvert () {
    int i;

    for (i=0; i<functions.size (); i++)
      ((FunctionEntry)(functions.elementAt (i))).rootNode.finalConvert ();
  }

  public void process () {
    int i;

    root.flatten ();
    root.beforeFunctionsSet ("CE");
    root.setFunctions (main);
    root.afterFunctionsSet ();
    removeDeclarations ();
    unLoop ();
    unCase ();
    unIf ();
    unNestFunctions ();
    unWith ();
    names ();
    casts ();
    byReference ();
    finalConvert ();
  }

  public void print () {
//    root.print (0);
    System.out.print (this);
  }

  public String toString () {
    String result="", typeDef, arrayPart, tempR;
    int i;
    ConstantEntry tempCE;
    TypeEntry tempTE;
    VariableEntry tempVE;
    FunctionEntry tempFE;

    result+="#include <stdlib.h>"+Auxiliary.newLine;
    result+="#include <stdio.h>"+Auxiliary.newLine;
    result+="#include <math.h>"+Auxiliary.newLine;
    result+=Auxiliary.newLine;
    for (i=0; i<constants.size (); i++) {
      tempCE=(ConstantEntry)(constants.elementAt (i));
      result+="#define "+tempCE.name+" "+tempCE.value+Auxiliary.newLine;
    }
    result+=Auxiliary.newLine;
    for (i=0; i<types.size (); i++) {
      tempTE=(TypeEntry)(types.elementAt (i));
      typeDef=tempTE.getDefinition ();
      if (typeDef.length ()>0)
        result+=typeDef+";"+Auxiliary.newLine;
    }
    result+=Auxiliary.newLine;
    for (i=0; i<main.variables.size (); i++) {
      tempVE=(VariableEntry)(main.variables.elementAt (i));
      arrayPart="";
      if (tempVE.type.type==Constants.TYPE_ARRAY)
        arrayPart=tempVE.type.arrayPart;
      result+=tempVE.type+" "+tempVE.name+" "+arrayPart+";"+Auxiliary.newLine;
    }
    result+=Auxiliary.newLine;
    for (i=1; i<functions.size (); i++) {
      tempFE=(FunctionEntry)(functions.elementAt (i));
        result+=tempFE.getHeader ()+";"+Auxiliary.newLine;
    }
    result+=Auxiliary.newLine;
    for (i=1; i<functions.size (); i++) {
      tempFE=(FunctionEntry)(functions.elementAt (i));
        result+=tempFE.getHeader ()+Auxiliary.newLine+tempFE.getBody ()+Auxiliary.newLine+Auxiliary.newLine;
    }
    tempR=main.getHeader ()+Auxiliary.newLine+main.getBody ();
    tempR=tempR.substring (4, tempR.lastIndexOf ("}"));
    result+="int"+tempR+"return 0;"+Auxiliary.newLine+"}";
    result+=Auxiliary.newLine;
    return result;
  }
}