package compiler;

import java.util.Vector;

class CTreeNode {

  private CTreeNode parent=null;
  private boolean terminal;

  public CTree tree;
  public int symbolNum;
  public String name;
  public Vector children;
  public String prefix;
  public FunctionEntry function;
  public int type;
  public TypeEntry varType;
  public int dataType;

  public CTreeNode (ParseTreeNode source, CTree tree1) {
    CTreeNode newChild;
    int i;

    tree=tree1;
    symbolNum=source.symbolNum;
    if (symbolNum==sym.NIZ)
      name=source.name.toString ();
    else
      name=source.name.toString ().toLowerCase ();
    if (symbolNum==sym.IME)
      name="P"+name;
    if (name.startsWith ("vec_"))
      type=Constants.LIST;
    else
      type=Constants.NONE;
    terminal=source.terminal;
    dataType=Constants.NONE;
    varType=null;
    children=new Vector ();
    if (source.children!=null)
      for (i=0; i<source.children.size (); i++) {
        newChild=new CTreeNode ((ParseTreeNode)(source.children.elementAt (i)), tree);
        addChild (newChild);
      }
  }

  public CTreeNode (int symbolNum1, String name1, CTree tree1) {
    tree=tree1;
    symbolNum=symbolNum1;
    name=name1;
    if (symbolNum>-1)
      terminal=true;
    else
      terminal=false;
    dataType=Constants.NONE;
    varType=null;
    children=new Vector ();
  }

  public void addChild (CTreeNode child) {
    child.parent=this;
    children.add (child);
  }

  public void addChildren (int position, Vector children1) {
    int i;
    CTreeNode tempCTN;

    for (i=0; i<children1.size (); i++) {
      tempCTN=(CTreeNode)(children1.elementAt (i));
      tempCTN.parent=this;
      children.add (position+i, tempCTN);
    }
  }

  public CTreeNode copy () {
    CTreeNode copied;
    int i;

    copied=new CTreeNode (symbolNum, name, tree);
    copied.parent=parent;
    copied.prefix=prefix;
    copied.function=function;
    copied.type=type;
    copied.varType=varType;
    copied.dataType=dataType;
    for (i=0; i<children.size (); i++)
      copied.addChild (((CTreeNode)(children.elementAt (i))).copy ());
    return copied;
  }

  public void flatten () {
    CTreeNode tempCTN;
    int i;

    if (type==Constants.LIST) {
      if (children.size ()==2)
        type=Constants.LIST_NON_SEPERATED;
      else
        type=Constants.LIST_SEPERATED;
      for (i=0; i<children.size (); i++)
        ((CTreeNode)(children.elementAt (i))).flatten ();
      tempCTN=((CTreeNode)(children.elementAt (0)));
      if ((tempCTN.children.size ()>0) && (Constants.common (tempCTN.type)==Constants.LIST)) {
        children.removeElementAt (0);
        addChildren (0, tempCTN.children);
      }
    } else
      for (i=0; i<children.size (); i++)
        ((CTreeNode)(children.elementAt (i))).flatten ();
  }

  public void beforeFunctionsSet (String prefix1) {
    int i;
    String newPrefix=new String (prefix1);

    prefix=prefix1;
    if ((name.compareTo ("def_procedure")==0) || (name.compareTo ("def_funkcije")==0))
      newPrefix+=((CTreeNode)(children.elementAt (1))).name+"E";
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).beforeFunctionsSet (newPrefix);
    if (name.compareTo ("def_konstante")==0)
      tree.constants.add (new ConstantEntry (this));
    if (name.compareTo ("def_tipa")==0)
      tree.types.add (new TypeEntry (this));
  }

  public void setFunctions (FunctionEntry function1) {
    int i;
    FunctionEntry newFunction=function1;

    function=function1;
    if ((name.compareTo ("def_procedure")==0) || (name.compareTo ("def_funkcije")==0)) {
      newFunction=new FunctionEntry (this);
      tree.functions.add (newFunction);
    }
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).setFunctions (newFunction);
  }

  public void afterFunctionsSet () {
    int i;

    if (name.compareTo ("def_spremenljivke")==0)
      VariableEntry.processVariables (this, function.variables);
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).afterFunctionsSet ();
  }

  public void removeDeclarations () {
    int i=0;

    while (i<children.size ())
      if ((((CTreeNode)(children.elementAt (i))).name.startsWith ("dekl_")) && (((CTreeNode)(children.elementAt (i))).name.compareTo ("dekl_stavkov")!=0))
        children.removeElementAt (i);
      else {
        ((CTreeNode)(children.elementAt (i))).removeDeclarations ();
        i++;
      }
  }

  public void unLoop () {
    int i;
    CTreeNode functionRoot, functionBody, ifStatement, condition, recursion, increment, tempCTN, tempCTN1, tempCTN2, tempCTN3, newLoop;

    if ((name.compareTo ("st_while")==0) || (name.compareTo ("st_repeat")==0) || (name.compareTo ("st_for")==0)) {
      functionRoot=new CTreeNode (-1, "def_procedure", tree);
      functionRoot.addChild (new CTreeNode (sym.PROCEDURE, "procedure", tree));
      if (name.compareTo ("st_while")==0)
        functionRoot.addChild (new CTreeNode (sym.IME, "Lwhile"+tree.loopCount, tree));
      if (name.compareTo ("st_repeat")==0)
        functionRoot.addChild (new CTreeNode (sym.IME, "Lrepeat"+tree.loopCount, tree));
      if (name.compareTo ("st_for")==0)
        functionRoot.addChild (new CTreeNode (sym.IME, "Lfor"+tree.loopCount, tree));
      tree.loopCount++;
      functionRoot.addChild (new CTreeNode (-1, "seznam parametrov", tree));
      functionRoot.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      tempCTN=new CTreeNode (-1, "blok", tree);
      tempCTN1=new CTreeNode (-1, "dekl_stavkov", tree);
      tempCTN1.addChild (new CTreeNode (sym.BEGIN, "begin", tree));
      functionBody=new CTreeNode (-1, "vec_stavkov", tree);
      if (name.compareTo ("st_while")==0)
        functionBody.addChild (((CTreeNode)(children.elementAt (3))).copy ());
      if (name.compareTo ("st_repeat")==0)
        functionBody.addChildren (functionBody.children.size (), ((CTreeNode)(children.elementAt (1))).copy ().children);
      if (name.compareTo ("st_for")==0)
        functionBody.addChild (((CTreeNode)(children.elementAt (7))).copy ());
      functionBody.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      if (name.compareTo ("st_for")==0) {
        increment=new CTreeNode (-1, "st_prirejanje", tree);
        tempCTN2=new CTreeNode (-1, "spremenljivka_ali_funkcija", tree);
        tempCTN3=new CTreeNode (-1, "spremenljivka", tree);
        tempCTN3.addChild (((CTreeNode)(children.elementAt (1))).copy ());
        tempCTN2.addChild (tempCTN3);
        increment.addChild (tempCTN2);
        increment.addChild (new CTreeNode (sym.PRIREJANJE, ":=", tree));
        tempCTN2=new CTreeNode (-1, "izraz", tree);
        tempCTN3=new CTreeNode (-1, "spremenljivka", tree);
        tempCTN3.addChild (((CTreeNode)(children.elementAt (1))).copy ());
        tempCTN2.addChild (tempCTN3);
        if (((CTreeNode)(((CTreeNode)(children.elementAt (4))).children.elementAt (0))).symbolNum==sym.TO)
          tempCTN2.addChild (new CTreeNode (sym.PLUS, "+", tree));
        else
          tempCTN2.addChild (new CTreeNode (sym.MINUS, "-", tree));
        tempCTN2.addChild (new CTreeNode (sym.NARAVNOSTEVILO, "1", tree));
        increment.addChild (tempCTN2);
        functionBody.addChild (increment);
        functionBody.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      }
      ifStatement=new CTreeNode (-1, "st_if", tree);
      ifStatement.addChild (new CTreeNode (sym.IF, "if", tree));
      if (name.compareTo ("st_while")==0)
        ifStatement.addChild (((CTreeNode)(children.elementAt (1))).copy ());
      if (name.compareTo ("st_repeat")==0) {
        condition=new CTreeNode (-1, "faktor", tree);
        condition.addChild (new CTreeNode (sym.NOT, "not", tree));
        tempCTN2=new CTreeNode (-1, "faktor", tree);
        tempCTN2.addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
        tempCTN2.addChild (((CTreeNode)(children.elementAt (3))).copy ());
        tempCTN2.addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree) );
        condition.addChild (tempCTN2);
        ifStatement.addChild (condition);
      }
      if (name.compareTo ("st_for")==0) {
        condition=new CTreeNode (-1, "izraz", tree);
        tempCTN2=new CTreeNode (-1, "spremenljivka", tree);
        tempCTN2.addChild (((CTreeNode)(children.elementAt (1))).copy ());
        condition.addChild (tempCTN2);
        if (((CTreeNode)(((CTreeNode)(children.elementAt (4))).children.elementAt (0))).symbolNum==sym.TO)
          condition.addChild (new CTreeNode (sym.MANJSEENAKO, "<=", tree));
        else
          condition.addChild (new CTreeNode (sym.VECJEENAKO, ">=", tree));
        condition.addChild (((CTreeNode)(children.elementAt (5))).copy ());
        ifStatement.addChild (condition);
      }
      ifStatement.addChild (new CTreeNode (sym.THEN, "then", tree));
      recursion=new CTreeNode (-1, "st_procedura", tree);
      recursion.addChild (new CTreeNode (sym.IME, ((CTreeNode)(functionRoot.children.elementAt (1))).name, tree));
      ifStatement.addChild (recursion);
      functionBody.addChild (ifStatement);
      tempCTN1.addChild (functionBody);
      tempCTN1.addChild (new CTreeNode (sym.END, "end", tree));
      tempCTN.addChild (tempCTN1);
      functionRoot.addChild (tempCTN);
      functionRoot.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      functionRoot.beforeFunctionsSet (prefix);
      functionRoot.setFunctions (function);
      newLoop=new CTreeNode (-1, "dekl_stavkov", tree);
      newLoop.addChild (new CTreeNode (sym.BEGIN, "begin", tree));
      tempCTN=new CTreeNode (-1, "vec_stavkov", tree);
      if (name.compareTo ("st_for")==0) {
        tempCTN1=new CTreeNode (-1, "st_prirejanje", tree);
        tempCTN2=new CTreeNode (-1, "spremenljivka_ali_funkcija", tree);
        tempCTN3=new CTreeNode (-1, "spremenljivka", tree);
        tempCTN3.addChild (((CTreeNode)(children.elementAt (1))).copy ());
        tempCTN2.addChild (tempCTN3);
        tempCTN1.addChild (tempCTN2);
        tempCTN1.addChild (((CTreeNode)(children.elementAt (2))).copy ());
        tempCTN1.addChild (((CTreeNode)(children.elementAt (3))).copy ());
        tempCTN.addChild (tempCTN1);
        tempCTN.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      }
      if ((name.compareTo ("st_while")==0) || (name.compareTo ("st_for")==0))
        tempCTN.addChild (ifStatement.copy ());
      if (name.compareTo ("st_repeat")==0)
        tempCTN.addChild (recursion.copy ());
      newLoop.addChild (tempCTN);
      newLoop.addChild (new CTreeNode (sym.END, "end", tree));
      newLoop.beforeFunctionsSet (prefix);
      newLoop.setFunctions (function);
      name="dekl_stavkov";
      children.removeAllElements ();
      addChild (newLoop);
    } else
      for (i=0; i<children.size (); i++)
        ((CTreeNode)(children.elementAt (i))).unLoop ();
  }

  public void unCase () {
    int i, j;
    CTreeNode condPart, caseList, caseItem, condList, caseStatement, statementList, ifStatement, ifCond, condItem, tempCTN;

    if (name.compareTo ("st_case")==0) {
      statementList=new CTreeNode (-1, "vec_stavkov", tree);
      condPart=(CTreeNode)(children.elementAt (1));
      caseList=(CTreeNode)(children.elementAt (3));
      for (i=0; i<caseList.children.size (); i+=2) {
        caseItem=(CTreeNode)(caseList.children.elementAt (i));
        condList=(CTreeNode)(caseItem.children.elementAt (0));
        ifStatement=new CTreeNode (-1, "st_if", tree);
        ifStatement.addChild (new CTreeNode (sym.IF, "if", tree));
        ifCond=new CTreeNode (-1, "izraz", tree);
        for (j=0; j<condList.children.size (); j+=2) {
          condItem=(CTreeNode)(condList.children.elementAt (j));
          tempCTN=new CTreeNode (-1, "izraz", tree);
          tempCTN.addChild (condPart.copy ());
          tempCTN.addChild (new CTreeNode (sym.ENACAJ, "=", tree));
          tempCTN.addChild (condItem.copy ());
          ifCond.addChild (tempCTN);
          if (j<condList.children.size()-1)
            ifCond.addChild (new CTreeNode (sym.OR, "or", tree));
        }
        ifStatement.addChild (ifCond);
        ifStatement.addChild (new CTreeNode (sym.THEN, "then", tree));
        caseStatement=(CTreeNode)(caseItem.children.elementAt (2));
        ifStatement.addChild (caseStatement.copy ());
        statementList.addChild (ifStatement);
      }
      children.removeAllElements ();
      name="dekl_stavkov";
      addChild (new CTreeNode (sym.BEGIN, "begin", tree));
      addChild (statementList);
      addChild (new CTreeNode (sym.END, "end", tree));
      beforeFunctionsSet (prefix);
      setFunctions (function);
    }
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).unCase ();
  }

  public void unIf () {
    int i;
    CTreeNode functionRoot, functionBody, newThen, newElse, tempCTN, tempCTN1, tempCTN2, tempCTN3, tempCTN4;

    if (name.compareTo ("st_if")==0) {
      functionRoot=new CTreeNode (-1, "def_funkcije", tree);
      functionRoot.addChild (new CTreeNode (sym.FUNCTION, "function", tree));
      functionRoot.addChild (new CTreeNode (sym.IME, "ITEthen"+tree.iteCount, tree));
      tree.iteCount++;
      functionRoot.addChild (new CTreeNode (-1, "seznam parametrov", tree));
      functionRoot.addChild (new CTreeNode (sym.DVOPICJE, ":", tree));
      functionRoot.addChild (new CTreeNode (sym.IME, "Pinteger", tree));
      functionRoot.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      tempCTN=new CTreeNode (-1, "blok", tree);
      tempCTN1=new CTreeNode (-1, "dekl_stavkov", tree);
      tempCTN1.addChild (new CTreeNode (sym.BEGIN, "begin", tree));
      functionBody=new CTreeNode (-1, "vec_stavkov", tree);
      functionBody.addChild (((CTreeNode)(children.elementAt (3))).copy ());
      tempCTN2=new CTreeNode (-1, "st_prirejanje", tree);
      tempCTN3=new CTreeNode (-1, "spremenljivka_ali_funkcija", tree);
      tempCTN4=new CTreeNode (-1, "spremenljivka", tree);
      tempCTN4.addChild (new CTreeNode (sym.IME, ((CTreeNode)(functionRoot.children.elementAt (1))).name, tree));
      tempCTN3.addChild (tempCTN4);
      tempCTN2.addChild (tempCTN3);
      tempCTN2.addChild (new CTreeNode (sym.PRIREJANJE, ":=", tree));
      tempCTN2.addChild (new CTreeNode (sym.NARAVNOSTEVILO, "0", tree));
      functionBody.addChild (tempCTN2);
      tempCTN1.addChild (functionBody);
      tempCTN1.addChild (new CTreeNode (sym.END, "end", tree));
      tempCTN.addChild (tempCTN1);
      functionRoot.addChild (tempCTN);
      functionRoot.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
      functionRoot.beforeFunctionsSet (prefix);
      functionRoot.setFunctions (function);
      newThen=new CTreeNode (-1, "faktor", tree);
      newThen.addChild (new CTreeNode (sym.IME, ((CTreeNode)(functionRoot.children.elementAt (1))).name, tree));
      if (children.size ()>5) {
        functionRoot=new CTreeNode (-1, "def_funkcije", tree);
        functionRoot.addChild (new CTreeNode (sym.FUNCTION, "function", tree));
        functionRoot.addChild (new CTreeNode (sym.IME, "ITEelse"+tree.iteCount, tree));
        tree.iteCount++;
        functionRoot.addChild (new CTreeNode (-1, "seznam parametrov", tree));
        functionRoot.addChild (new CTreeNode (sym.DVOPICJE, ":", tree));
        functionRoot.addChild (new CTreeNode (sym.IME, "Pinteger", tree));
        functionRoot.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
        tempCTN=new CTreeNode (-1, "blok", tree);
        tempCTN1=new CTreeNode (-1, "dekl_stavkov", tree);
        tempCTN1.addChild (new CTreeNode (sym.BEGIN, "begin", tree));
        functionBody=new CTreeNode (-1, "vec_stavkov", tree);
        functionBody.addChild (((CTreeNode)(children.elementAt (5))).copy ());
        tempCTN2=new CTreeNode (-1, "st_prirejanje", tree);
        tempCTN3=new CTreeNode (-1, "spremenljivka_ali_funkcija", tree);
        tempCTN4=new CTreeNode (-1, "spremenljivka", tree);
        tempCTN4.addChild (new CTreeNode (sym.IME, ((CTreeNode)(functionRoot.children.elementAt (1))).name, tree));
        tempCTN3.addChild (tempCTN4);
        tempCTN2.addChild (tempCTN3);
        tempCTN2.addChild (new CTreeNode (sym.PRIREJANJE, ":=", tree));
        tempCTN2.addChild (new CTreeNode (sym.NARAVNOSTEVILO, "0", tree));
        functionBody.addChild (tempCTN2);
        tempCTN1.addChild (functionBody);
        tempCTN1.addChild (new CTreeNode (sym.END, "end", tree));
        tempCTN.addChild (tempCTN1);
        functionRoot.addChild (tempCTN);
        functionRoot.addChild (new CTreeNode (sym.PODPICJE, ";", tree));
        functionRoot.beforeFunctionsSet (prefix);
        functionRoot.setFunctions (function);
        newElse=new CTreeNode (-1, "faktor", tree);
        newElse.addChild (new CTreeNode (sym.IME, ((CTreeNode)(functionRoot.children.elementAt (1))).name, tree));
      } else
        newElse=new CTreeNode (sym.NARAVNOSTEVILO, "0", tree);
      tempCTN=((CTreeNode)(children.elementAt (1))).copy ();
      children.removeAllElements ();
      addChild (tempCTN);
      addChild (new CTreeNode (sym.THEN, "?", tree));
      addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
      addChild (newThen);
      addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
      addChild (new CTreeNode (sym.ELSE, ":", tree));
      addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
      addChild (newElse);
      addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
      beforeFunctionsSet (prefix);
      setFunctions (function);
    } else
      for (i=0; i<children.size (); i++)
        ((CTreeNode)(children.elementAt (i))).unIf ();
  }

  public void unNestFunctions () {
    FunctionEntry tempFE;
    ParameterEntry currentParam;
    CTreeNode tempCTN, tempCTN1;
    int currentParamCount, newParamCount, i;

    if (((name.compareTo ("spremenljivka")==0) && ((parent.name.compareTo ("spremenljivka_ali_funkcija")!=0))) || (name.compareTo ("faktor")==0) || (name.compareTo ("st_procedura")==0)) {
      tempCTN=(CTreeNode)(children.elementAt (0));
      if (tempCTN.symbolNum==sym.IME) {
        tempFE=tree.lookupFunction (tempCTN.name, prefix);
        if (tempFE!=null) {
//          tempCTN.name=tempFE.name; // Najbrz se to uredi v spodnji metodi.
          if (children.size ()>1) {
            tempCTN=(CTreeNode)(children.elementAt (2));
            currentParamCount=((tempCTN.children.size ()+1)/2);
          } else
            currentParamCount=0;
          newParamCount=tempFE.parameters.size ();
          if (newParamCount>currentParamCount) {
            if (currentParamCount==0) {
              addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
              addChild (new CTreeNode (-1, "vec_parametrov", tree));
              addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
              tempCTN=(CTreeNode)(children.elementAt (2));
            }
            for (i=currentParamCount; i<newParamCount; i++) {
              currentParam=(ParameterEntry)(tempFE.parameters.elementAt (i));
              if (tempCTN.children.size ()>0)
                tempCTN.addChild (new CTreeNode (sym.VEJICA, ",", tree));
              tempCTN1=new CTreeNode (-1, "izraz", tree);
              tempCTN1.addChild (new CTreeNode (sym.IME, tree.lookupVariable (Auxiliary.getName (currentParam.name), function).name, tree));
              tempCTN.addChild (tempCTN1);
            }
            beforeFunctionsSet (prefix);
            setFunctions (function);
          }
        }
      }
    }
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).unNestFunctions ();
  }

  public void unWith (Vector withNames) {
    int i, j;
    CTreeNode addName, additions, tempCTN, tempCTN1;
    Vector processedWithNames;

    if ((((name.compareTo ("spremenljivka")==0) && (((CTreeNode)(children.elementAt (0))).name.compareTo ("spremenljivka_z_dodatki")!=0)) || (name.compareTo ("spremenljivka_z_dodatki")==0)) && (withNames.size ()>0)) {
      processedWithNames=new Vector ();
      for (i=0; i<withNames.size (); i++) {
        tempCTN=(CTreeNode)(((CTreeNode)(withNames.elementAt (i))).children.elementAt (0));
        if (tempCTN.symbolNum==sym.IME)
          processedWithNames.add (tempCTN);
        else {
          processedWithNames.add ((CTreeNode)(tempCTN.children.elementAt (0)));
          tempCTN1=(CTreeNode)(tempCTN.children.elementAt (1));
          for (j=0; j<tempCTN1.children.size (); j++)
            processedWithNames.add ((CTreeNode)(tempCTN1.children.elementAt (j)));
        }
      }
      addName=(CTreeNode)(processedWithNames.elementAt (0));
      additions=new CTreeNode (-1, "vec_dodatkov_k_spremenljivki", tree);
      for (i=1; i<processedWithNames.size (); i++) {
        tempCTN=(CTreeNode)(processedWithNames.elementAt (i));
        if (tempCTN.symbolNum==sym.IME) {
          tempCTN1=new CTreeNode (-1, "dodatek_k_spremenljivki", tree);
          tempCTN1.addChild (new CTreeNode (sym.PIKA, ".", tree));
          tempCTN1.addChild (tempCTN);
          additions.addChild (tempCTN1);
        } else
          additions.addChild (tempCTN);
      }
      tempCTN=new CTreeNode (-1, "dodatek_k_spremenljivki", tree);
      tempCTN.addChild (new CTreeNode (sym.PIKA, ".", tree));
      tempCTN.addChild (((CTreeNode)(children.elementAt (0))).copy ());
      additions.addChild (tempCTN);
      if (name.compareTo ("spremenljivka_z_dodatki")==0) {
        tempCTN1=(CTreeNode)(children.elementAt (1));
        for (i=0; i<tempCTN1.children.size (); i++)
          additions.addChild (((CTreeNode)(tempCTN1.children.elementAt (i))).copy ());
      }
      children.removeAllElements ();
      name="spremenljivka_z_dodatki";
      addChild (addName);
      addChild (additions);
      beforeFunctionsSet (prefix);
      setFunctions (function);
    }
    if (name.compareTo ("st_with")==0) {
      tempCTN=(CTreeNode)(((CTreeNode)(children.elementAt (1))).children.elementAt(0));
      withNames.add (tempCTN.copy ());
      ((CTreeNode)(children.elementAt (3))).unWith (withNames);
      withNames.removeElementAt (withNames.size ()-1);
      tempCTN=((CTreeNode)(children.elementAt (3))).copy ();
      name="dekl_stavkov";
      children.removeAllElements ();
      addChild (new CTreeNode (sym.BEGIN, "begin", tree));
      addChild (tempCTN);
      addChild (new CTreeNode (sym.END, "end", tree));
      beforeFunctionsSet (prefix);
      setFunctions (function);
    } else
      for (i=0; i<children.size (); i++)
        ((CTreeNode)(children.elementAt (i))).unWith (withNames);
  }

  public void names () {
    VariableEntry tempVE=null;
    FunctionEntry tempFE=null;
    ConstantEntry tempCE=null;
    CTreeNode additions, indices, index, tempCTN, tempCTN1;
    int i, j;
    boolean alternate;
    Vector names;

    if (symbolNum==sym.IME) {
      if ((parent.name.compareTo ("spremenljivka")==0) || (parent.name.compareTo ("konstanta")==0))
        tempCE=tree.lookupConstant (name, prefix);
      if (tempCE!=null) {
        name=tempCE.name;
        type=Constants.VAR_CONSTANT;
        dataType=Auxiliary.getDataType ((CTreeNode)(tempCE.node.children.elementAt (2)));
      } else {
        if ((parent.name.compareTo ("spremenljivka")==0) || (parent.name.compareTo ("faktor")==0) || (parent.name.compareTo ("st_procedura")==0))
          tempFE=tree.lookupFunction (name, prefix);
        if (tempFE!=null) {
          name=tempFE.name;
          type=Constants.VAR_FUNCTION;
          varType=tempFE.type;
          dataType=Auxiliary.getDataType (varType);
          if ((tempFE.parameters.size ()==0) && (parent.parent.name.compareTo ("spremenljivka_ali_funkcija")!=0)) {
            parent.addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
            parent.addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
            parent.beforeFunctionsSet (parent.prefix);
            parent.setFunctions (parent.function);
          }
        } else {
          if ((parent.name.compareTo ("spremenljivka")==0) || (parent.name.compareTo ("spremenljivka_z_dodatki")==0) || (parent.name.compareTo ("st_for")==0))
            tempVE=tree.lookupVariable (name, function);
          if (tempVE!=null) {
            name=tempVE.name;
            if (parent.name.compareTo ("spremenljivka_z_dodatki")==0) {
              names=new Vector ();
              names.add (name);
              additions=(CTreeNode)(parent.children.elementAt (1));
              for (i=0; i<additions.children.size (); i++) {
                tempCTN=(CTreeNode)(additions.children.elementAt (i));
                if (tempCTN.children.size ()==3) {
                  indices=(CTreeNode)(tempCTN.children.elementAt (1));
                  alternate=false;
                  for (j=0; j<indices.children.size (); j++) {
                    index=(CTreeNode)(indices.children.elementAt (j));
                    if (alternate) {
                      index.name="][";
                      alternate=false;
                    }
                    else {
                      index.addChild (new CTreeNode (sym.MINUS, "-", tree));
                      index.addChild (new CTreeNode (sym.NARAVNOSTEVILO, tempVE.type.arrayMins.elementAt ((int)(j/2)).toString (), tree));
                      index.beforeFunctionsSet (index.prefix);
                      index.setFunctions (index.function);
                      alternate=true;
                    }
                  }
                }
                if (tempCTN.children.size ()==2) {
                  tempCTN1=(CTreeNode)(tempCTN.children.elementAt (1));
                  names.add (tempCTN1.name);
                  tempVE=tree.lookupVariable (names, function);
                }
              }
            }
            type=Constants.VAR_VARIABLE;
            varType=tempVE.type;
            dataType=Auxiliary.getDataType (varType);
          }
        }
      }
    }
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).names ();
  }

  public void casts () {
    int i, j;
    CTreeNode newFactor, left, middle, right, tempCTN;

    if (symbolNum==sym.DELJENO) {
      j=0;
      while ((CTreeNode)(parent.parent.children.elementAt (j+1))!=parent)
        j++;
      left=(CTreeNode)(parent.parent.children.elementAt (j));
      middle=(CTreeNode)(parent.parent.children.elementAt (j+1));
      right=(CTreeNode)(parent.parent.children.elementAt (j+2));
      if ((Auxiliary.getDataType (left)==Constants.DATA_INT) && (Auxiliary.getDataType (right)==Constants.DATA_INT)) {
        newFactor=new CTreeNode (-1, "faktor", tree);
        tempCTN=new CTreeNode (-1, "faktor", tree);
        tempCTN.addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
        tempCTN.addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
        tempCTN.addChild (new CTreeNode (sym.IME, "Preal", tree));
        tempCTN.addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
        tempCTN.addChild (left.copy ());
        tempCTN.addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
        tempCTN.dataType=Constants.DATA_DOUBLE;
        newFactor.addChild (tempCTN);
        newFactor.addChild (middle.copy ());
        tempCTN=new CTreeNode (-1, "faktor", tree);
        tempCTN.addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
        tempCTN.addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
        tempCTN.addChild (new CTreeNode (sym.IME, "Preal", tree));
        tempCTN.addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
        tempCTN.addChild (right.copy ());
        tempCTN.addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
        tempCTN.dataType=Constants.DATA_DOUBLE;
        newFactor.addChild (tempCTN);
        newFactor.parent=parent.parent;
        newFactor.dataType=Constants.DATA_DOUBLE;
        parent.parent.children.removeElementAt (j);
        parent.parent.children.removeElementAt (j);
        parent.parent.children.removeElementAt (j);
        parent.parent.children.insertElementAt (newFactor, j);
        parent.parent.beforeFunctionsSet (parent.prefix);
        parent.parent.setFunctions (parent.function);
      }
    }
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).casts ();
  }

  public void byReference (String paramOf, int paramNum, int mode) {
    int i, newParamNum=paramNum, newMode=mode;
    ParameterEntry tempPE;
    String newParamOf=new String (paramOf);
    FunctionEntry tempFE;
    CTreeNode tempCTN;

    if (symbolNum==sym.IME)
      for (i=0; i<function.parameters.size (); i++) {
        tempPE=(ParameterEntry)(function.parameters.elementAt (i));
        if ((tempPE.name.compareTo (name)==0) && (tempPE.paramType==Constants.PARAM_BY_REFERENCE))
          name="(*("+name+"))";
      }
    if (name.compareTo ("izraz")==0) {
      if (paramNum>-1) {
        tempFE=tree.lookupFunction (paramOf, prefix);
        if ((tempFE!=null) && (((ParameterEntry)(tempFE.parameters.elementAt (paramNum))).paramType==Constants.PARAM_BY_REFERENCE) && (mode==Constants.PARAM)) {
          tempCTN=copy ();
          children.removeAllElements ();
          addChild (new CTreeNode (sym.IME, "&", tree));
          addChild (tempCTN);
          beforeFunctionsSet (prefix);
          setFunctions (function);
          newMode=Constants.NONE;
        }
      }
    }
    for (i=0; i<children.size (); i++) {
      if ((name.compareTo ("st_procedura")==0) || ((name.compareTo ("faktor")==0) && (children.size ()==4)) || ((name.compareTo ("spremenljivka")==0) && (children.size ()==4))) {
        newParamOf=((CTreeNode)(children.elementAt (0))).name;
        newMode=Constants.PARAM;
      }
      if ((name.compareTo ("vec_parametrov")==0) || ((name.compareTo ("vec_izrazov")==0) && (parent.name.compareTo ("faktor")==0)))
        newParamNum=(int)(i/2);
      ((CTreeNode)(children.elementAt (i))).byReference (newParamOf, newParamNum, newMode);
    }
  }

  public void finalConvert () {
    int i, j, paramType;
    CTreeNode mallocVar, mallocType, tempCTN, tempCTN1, tempCTN2;
    String tempP, functionName, atBeginning;
    Vector recordVar;

    switch (symbolNum) {
      case sym.AND:
        name="&&";
      break;
      case sym.BEGIN:
        name="{";
      break;
      case sym.DIV:
        name="/";
      break;
      case sym.ENACAJ:
        name="==";
      break;
      case sym.END:
        name="}";
      break;
      case sym.IME:
        if (name.compareTo ("Pchar")==0)
          name="char";
        if (name.compareTo ("Pinteger")==0)
          name="int";
        if (name.compareTo ("Preal")==0)
          name="double";
        if (name.compareTo ("Pboolean")==0)
          name="int";
        if (name.compareTo ("Ptrue")==0) {
          symbolNum=sym.NARAVNOSTEVILO;
          name="1";
        }
        if (name.compareTo ("Pfalse")==0) {
          symbolNum=sym.NARAVNOSTEVILO;
          name="0";
        }
        if (name.compareTo ("Psin")==0)
          name="sin";
        if (name.compareTo ("Pcos")==0)
          name="cos";
        if (name.compareTo ("Pln")==0)
          name="log";
      break;
      case sym.MOD:
        name="%";
      break;
      case sym.NIL:
        name="NULL";
      break;
      case sym.NIZ:
        if (name.length ()>3)
          name="\""+name.substring (1, name.length ()-1)+"\"";
      break;
      case sym.NOT:
        name="!";
      break;
      case sym.OR:
        name="||";
      break;
      case sym.PACKED:
        name="";
      break;
      case sym.PRIREJANJE:
        name="=";
      break;
      case -1:
        if ((name.compareTo ("izraz")==0) && (parent!=null) && (parent.name.compareTo ("faktor")!=0)) {
          tempCTN=copy ();
          name="faktor";
          children.removeAllElements ();
          addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
          addChild (tempCTN);
          addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
          beforeFunctionsSet (prefix);
          setFunctions (function);
        }
        if (name.compareTo ("tip")==0) {
          tempCTN=(CTreeNode)(children.elementAt(0));
          if (tempCTN.name.compareTo ("enostaven_tip")==0) {
            tempCTN1=(CTreeNode)(tempCTN.children.elementAt (0));
            if (tempCTN1.symbolNum==sym.OKLEPAJ) {
              tempCTN1.name="{";
              tempCTN1=(CTreeNode)(tempCTN.children.elementAt (2));
              tempCTN1.name="}";
            }
          }
        }
        if (name.compareTo ("st_prirejanje")==0) {
          tempCTN=(CTreeNode)(children.elementAt (0));
          if (tempCTN.toString ().compareTo (function.name)==0) {
            tempCTN.name="return";
            tempCTN.terminal=true;
            tempCTN.children.removeAllElements ();
            children.removeElementAt (1);
          }
        }
        if (name.compareTo ("st_procedura")==0) {
          tempCTN=(CTreeNode)(children.elementAt (0));
          if ((tempCTN.name.compareTo ("Pwriteln")==0) || (tempCTN.name.compareTo ("Preadln")==0)) {
            if (tempCTN.name.compareTo ("Pwriteln")==0) {
              functionName="printf";
              tempP="\\n\", ";
            } else {
              functionName="scanf";
              tempP="\", ";
            }
            if (children.size ()==1)
              tempCTN.name=functionName+" (\"\\n\")";
            else {
              tempCTN.name=functionName+" (\"";
              tempCTN1=(CTreeNode)(children.elementAt (2));
              for (i=0; i<tempCTN1.children.size (); i+=2) {
                tempCTN2=(CTreeNode)(tempCTN1.children.elementAt (i));
                paramType=Auxiliary.getDataType (tempCTN2);
                if (paramType==Constants.DATA_INT)
                  tempCTN.name+="%d";
                if (paramType==Constants.DATA_DOUBLE)
                  tempCTN.name+="%lf";
                if (paramType==Constants.DATA_CHAR)
                  tempCTN.name+="%c";
                if (paramType==Constants.DATA_STRING)
                  tempCTN.name+="%s";
                tempCTN2.finalConvert ();
                if (functionName.compareTo ("scanf")==0)
                  tempP+="&";
                tempP+=tempCTN2+", ";
              }
              tempCTN.name+=tempP.substring (0, tempP.length ()-2)+")";
              children.removeElementAt (1);
              children.removeElementAt (1);
              children.removeElementAt (1);
            }
          }
          if (tempCTN.name.compareTo ("Pgetmem")==0) {
            tempCTN1=(CTreeNode)(children.elementAt (2));
            mallocVar=((CTreeNode)(tempCTN1.children.elementAt (0))).copy ();
            tempCTN2=((CTreeNode)(tempCTN1.children.elementAt (2))).copy ();
            children.removeAllElements ();
            while (tempCTN2.name.compareTo ("faktor")!=0)
              tempCTN2=(CTreeNode)(tempCTN2.children.elementAt (0));
            mallocType=(CTreeNode)(tempCTN2.children.elementAt (2));
            while (mallocType.symbolNum!=sym.IME)
              mallocType=(CTreeNode)(mallocType.children.elementAt (0));
            mallocType.name=tree.lookupType (mallocType, false).toString ();
            name="st_prirejanje";
            addChild (mallocVar);
            addChild (new CTreeNode (sym.PRIREJANJE, "=", tree));
            addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
            addChild (mallocType);
            addChild (new CTreeNode (sym.IME, "*", tree));
            addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
            addChild (new CTreeNode (sym.IME, "malloc", tree));
            addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
            addChild (new CTreeNode (sym.IME, "sizeof", tree));
            addChild (new CTreeNode (sym.OKLEPAJ, "(", tree));
            addChild (mallocType);
            addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
            addChild (new CTreeNode (sym.ZAKLEPAJ, ")", tree));
            beforeFunctionsSet (prefix);
            setFunctions (function);
          }
        }
        if (name.compareTo ("tip")==0) {
          tempCTN=(CTreeNode)(children.elementAt (0));
          if (tempCTN.name.compareTo ("^")==0) {
            tempCTN.name="*";
            addChild (tempCTN);
            children.removeElementAt (0);
            tempCTN=(CTreeNode)(children.elementAt (0));
            tempCTN.name=tree.lookupType (tempCTN, true).toString();

          }
          if (tempCTN.name.compareTo ("record")==0) {
            tempCTN=((CTreeNode)(children.elementAt (1))).copy ();
            children.removeAllElements ();
            addChild (new CTreeNode (sym.RECORD, "struct", tree));
            addChild (new CTreeNode (sym.BEGIN, "{", tree));
            addChild (tempCTN);
            addChild (new CTreeNode (sym.END, "}", tree));
            beforeFunctionsSet (prefix);
            setFunctions (function);
          }
        }
        if (name.compareTo ("zapis")==0) {
          recordVar=new Vector ();
          VariableEntry.processVariables (this, recordVar);
          if (recordVar.size ()>0) {
            children.removeAllElements ();
            addChild (new CTreeNode (sym.IME, ((VariableEntry)(recordVar.elementAt (0))).type.toString (), tree));
            for (i=0; i<recordVar.size (); i++) {
              tempCTN=new CTreeNode (sym.IME, ((VariableEntry)(recordVar.elementAt (i))).name, tree);
              tempCTN.varType=((VariableEntry)(recordVar.elementAt (0))).type;
              addChild (tempCTN);
              addChild (new CTreeNode (sym.IME, ((VariableEntry)(recordVar.elementAt (0))).type.arrayPart, tree));
              if (i<recordVar.size ()-1)
                addChild (new CTreeNode (sym.VEJICA, ",", tree));
            }
            beforeFunctionsSet (prefix);
            setFunctions (function);
            name="zapis1";
          }
        }
        if (name.compareTo ("tabela")==0) {
          while (children.size ()>1)
            children.removeElementAt (0);
        }
        if (name.compareTo ("spremenljivka_z_dodatki")==0) {
          tempCTN=(CTreeNode)(children.elementAt (1));
          atBeginning="";
          for (i=0; i<tempCTN.children.size (); i++) {
            tempCTN1=(CTreeNode)(tempCTN.children.elementAt (i));
            tempCTN2=(CTreeNode)(tempCTN1.children.elementAt (0));
            if (tempCTN2.symbolNum==sym.PUSCICA) {
              tempCTN2.name="))";
              atBeginning="(*("+atBeginning;
            }
          }
          tempCTN=(CTreeNode)(children.elementAt (0));
          tempCTN.name=atBeginning+tempCTN.name;
        }
        if (name.compareTo ("vec_stavkov")==0) {
          i=0;
          while (i<children.size ()) {
            tempCTN=(CTreeNode)(children.elementAt (i));
            if (tempCTN.symbolNum==sym.PODPICJE)
              children.removeElementAt (i);
            else
              i++;
          }
        }
        if ((name.compareTo ("stavek")==0) && (children.size ()>1)) {
          children.removeElementAt (0);
          children.removeElementAt (0);
        }
        if (name.startsWith ("st_")) {
          addChild (new CTreeNode (sym.PODPICJE, ";", tree));
          beforeFunctionsSet (prefix);
          setFunctions (function);
        }
      break;
    }
    for (i=0; i<children.size (); i++)
      ((CTreeNode)(children.elementAt (i))).finalConvert ();
  }

  public void print (int identSize) {
    int i, j;
    String ident="";

    for (j=0; j<identSize; j++)
      ident=ident+" ";
    if (terminal)
      System.out.print (ident+"\""+name+"\"");
    if (children.size ()>0) {
      System.out.println (" {");
      for (i=0; i<children.size (); i++)
        ((CTreeNode)(children.elementAt (i))).print (identSize+2);
      System.out.println (ident+"}");
    } else
      System.out.println ();
  }

  public String toString () {
    int i;
    String string, tempS;

    string="";
    if (children.size ()>0)
      for (i=0; i<children.size (); i++) {
        tempS=((CTreeNode)(children.elementAt (i))).toString ();
        if (tempS.length ()>0)
          if (string.length ()>0)
            string+=" "+tempS;
          else
            string=tempS;
      }
    if (string.length ()>0) {
      if (terminal)
        if ((symbolNum==sym.PODPICJE) || (symbolNum==sym.BEGIN))
          string=name+Auxiliary.newLine+string;
        else
          string=name+" "+string;
    } else
      if (terminal)
        if ((symbolNum==sym.PODPICJE) || (symbolNum==sym.BEGIN))
          string=name+"\r\n";
        else
          string=name;
    return string;
  }
}