package compiler;

import java.util.Vector;

class ParseTreeNode {

  private ParseTreeNode parent;
  private String comment;

  public int symbolNum, lineNum, charNum;
  public Object name;
  public boolean terminal;
  public Vector children;
  public String hLinkFrom, hLinkTo;

  public ParseTreeNode (int symbolNum1, Object name1, int lineNum1, int charNum1) {
    symbolNum=symbolNum1;
    name=name1;
    terminal=true;
    lineNum=lineNum1;
    charNum=charNum1;
    hLinkFrom="";
    hLinkTo="";
    comment="";
  }

  public ParseTreeNode (Object name1) {
    symbolNum=-1;
    name=name1;
    terminal=false;
    children=new Vector ();
    lineNum=-1;
    charNum=-1;
    hLinkFrom="";
    hLinkTo="";
    comment="";
  }

  public ParseTreeNode (Object name1, ParseTreeNode locator, ParseTree tree) {
    this (name1);
    lineNum=locator.lineNum;
    charNum=locator.charNum;
    tree.addLink (this);
  }

  public void addChild (ParseTreeNode child) {
    if (child!=null) {
      child.parent=this;
      children.add (child);
    }
  }

  public void addComment (String comment1) {
    if (comment.length ()>0)
      comment+=Auxiliary.spacePP;
    comment+="{"+comment1+"}";
  }

  public void print (int indentSize) {
    int i, j;
    String indent="";

    for (j=0; j<indentSize; j++)
      indent=indent+" ";
    if (terminal)
      System.out.print (indent+"\""+name+"\"");
    else
      System.out.print (indent+name);
    if ((!terminal) && (lineNum>=0)) System.out.print (" ("+lineNum+", "+charNum+")");
    if ((children!=null) && (children.size ()>0)) {
      System.out.println (" {");
      for (i=0; i<children.size (); i++)
        ((ParseTreeNode)(children.elementAt (i))).print (indentSize+2);
      System.out.println (indent+"}");
    } else
      System.out.println ();
  }

  public Vector prettyPrint (ParseTree tree, int indentSize, int step, boolean align, int spaceMode, int spaceCount) {
    int i, tempI, newIndentSize=indentSize, newSpaceMode=spaceMode, newSpaceCount=spaceCount;
    String indent="", reducedIndent="", extendedIndent="", spaces="";
    Vector output=new Vector ();

    for (i=0; i<indentSize; i++)
      indent+=Auxiliary.spacePP;
    for (i=0; i<indentSize-step; i++)
      reducedIndent+=Auxiliary.spacePP;
    for (i=0; i<indentSize+step; i++)
      extendedIndent+=Auxiliary.spacePP;
    for (i=0; i<spaceCount; i++)
      spaces+=Auxiliary.spacePP;
    if (terminal) {
      if ((symbolNum==sym.NARAVNOSTEVILO) && (((String)(parent.name)).compareTo ("stavek")==0)) {
        symbolNum=Constants.NODE_NUM_LABEL;
        tempI=name.toString ().length ();
        if (tempI>tree.maxLabel)
          tree.maxLabel=tempI;
      }
      if ((symbolNum==sym.DVOPICJE) && (((String)(parent.name)).compareTo ("stavek")==0))
        symbolNum=Constants.NODE_COLON_LABEL;
      if (symbolNum==sym.END) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
      }
      if ((spaceMode==Constants.SPACES_EQUALS) && (symbolNum==sym.ENACAJ))
        output.add (new ParseTreeNode (Constants.NODE_PP, spaces, -1, -1));
      if ((spaceMode==Constants.SPACES_COLON) && (symbolNum==sym.DVOPICJE))
        output.add (new ParseTreeNode (Constants.NODE_PP, spaces, -1, -1));
      output.add (this);
      if (comment.length ()>0)
        output.add (new ParseTreeNode (Constants.NODE_COMMENT, comment, -1, -1));
      if ((symbolNum==sym.PODPICJE) && (((String)(parent.name)).compareTo ("vec_elementov_seznama_parametrov")!=0) && (((((String)(parent.name)).compareTo("vec_stavkov")!=0) && (((String)(parent.name)).compareTo("vec_elementov_st_case")!=0) && (((String)(parent.name)).compareTo("vec_zapisov")!=0) && (((String)(parent.name)).compareTo("vec_elementov_dela_s_case")!=0)) || ((((ParseTreeNode)(parent.children.elementAt (parent.children.size ()-1))).children!=null) && (((ParseTreeNode)(parent.children.elementAt (parent.children.size ()-1))).children.size()!=0)))) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        if (((((String)(parent.name)).compareTo ("def_procedure")==0) || (((String)(parent.name)).compareTo ("def_funkcije")==0)) && (this!=parent.children.elementAt (parent.children.size ()-1)) && (((ParseTreeNode)(parent.children.elementAt (parent.children.size ()-2))).children.size()<2))
          output.add (new ParseTreeNode (Constants.NODE_PP, reducedIndent, -1, -1));
        else
          output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
      }
      if ((symbolNum==sym.BEGIN) || ((symbolNum==sym.OF) && (((String)(parent.name)).endsWith ("case"))) || (symbolNum==sym.RECORD)) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
      }
      if ((symbolNum==sym.LABEL) || (symbolNum==sym.CONST) || (symbolNum==sym.TYPE) || ((symbolNum==sym.VAR) && (((String)(parent.name)).compareTo("sp_po_referenci")!=0))) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
      }
    } else {
      if (align && ((((String)name).compareTo ("dekl_konstant")==0) || (((String)name).compareTo ("dekl_tipov")==0) || (((String)name).compareTo ("dekl_spremenljivk")==0) || (((String)name).compareTo ("seznam_komponent")==0) || (((String)name).compareTo ("st_case")==0) || (((String)name).compareTo ("del_s_case")==0)))
        newSpaceCount=spaces (tree, step, align);
      if (align && ((((String)name).compareTo ("def_konstante")==0) || (((String)name).compareTo ("def_tipa")==0))) {
        newSpaceMode=Constants.SPACES_EQUALS;
        newSpaceCount=spaceCount-((ParseTreeNode)(children.elementAt (0))).prettyPrintFinal (tree, step, align, false, false, false).length();
      }
      if (align && ((((String)name).compareTo ("def_spremenljivke")==0) || (((String)name).compareTo ("zapis")==0) || (((String)name).compareTo ("element_st_case")==0) || (((String)name).compareTo ("element_dela_s_case")==0))) {
        newSpaceMode=Constants.SPACES_COLON;
        newSpaceCount=spaceCount-((ParseTreeNode)(children.elementAt (0))).prettyPrintFinal (tree, step, align, false, false, false).length();
      }
    }
    if (children!=null)
      for (i=0; i<children.size (); i++) {
        if ((((String)name).compareTo ("dekl_oznak")==0) || (((String)name).compareTo ("dekl_konstant")==0) || (((String)name).compareTo ("dekl_tipov")==0) || (((String)name).compareTo ("dekl_spremenljivk")==0) || (((String)name).compareTo ("dekl_stavkov")==0))
          if (i<2)
            newIndentSize=indentSize+step;
          else
            newIndentSize=indentSize;
        if ((((String)name).compareTo ("tip")==0) && (children.size ()==3) && (((ParseTreeNode)(children.elementAt (0))).symbolNum==sym.RECORD))
          if (i<2)
            newIndentSize=indentSize+step;
          else
            newIndentSize=indentSize;
        if (((String)name).compareTo ("st_case")==0)
          if ((i>1) && (i<4))
            newIndentSize=indentSize+step;
          else
            newIndentSize=indentSize;
        if ((((String)name).compareTo ("def_procedure")==0) || (((String)name).compareTo ("def_funkcije")==0))
          newIndentSize=indentSize+step;
        if (((String)name).compareTo ("blok")==0)
          if (((String)(((ParseTreeNode)(children.elementAt (i))).name)).compareTo ("dekl_stavkov")==0)
            newIndentSize=Math.max (0, indentSize-step);
          else
            newIndentSize=indentSize;
        if (((String)name).compareTo ("st_repeat")==0) {
          if (i==1)
            newIndentSize=indentSize+step;
          else
            newIndentSize=indentSize;
          if ((((ParseTreeNode)(children.elementAt (1))).children.size ()>1) && ((((ParseTreeNode)(((ParseTreeNode)(children.elementAt (1))).children.elementAt (2))).children.size ()>0) || (((String)(((ParseTreeNode)(((ParseTreeNode)(((ParseTreeNode)(children.elementAt (1))).children.elementAt (0))).children.elementAt (0))).name)).compareTo ("vec_stavkov")==0))) {
            if (i==1) {
              output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
              output.add (new ParseTreeNode (Constants.NODE_PP, extendedIndent, -1, -1));
            }
            if (i==2) {
              output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
              output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
            }
          }
        }
        output.addAll (((ParseTreeNode)(children.elementAt (i))).prettyPrint (tree, newIndentSize, step, align, newSpaceMode, newSpaceCount));
      }
    if (!terminal) {
      if ((((String)name).compareTo ("dekl_oznak")==0) || (((String)name).compareTo ("dekl_konstant")==0) || (((String)name).compareTo ("dekl_tipov")==0) || (((String)name).compareTo ("dekl_spremenljivk")==0)) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        if ((parent.children.size ()>1) && (this==(ParseTreeNode)(parent.children.elementAt (parent.children.size ()-2))))
          output.add (new ParseTreeNode (Constants.NODE_PP, reducedIndent, -1, -1));
        else
          output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
      }
      if (((String)name).compareTo ("dekl_procedur_in_funkcij")==0) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        output.add (new ParseTreeNode (Constants.NODE_PP, reducedIndent, -1, -1));
      }
      if ((((String)name).compareTo ("def_procedure_ali_funkcije")==0) && (((String)(parent.parent.name)).compareTo ("dekl_procedur_in_funkcij")!=0)) {
        output.add (new ParseTreeNode (Constants.NODE_PP, Auxiliary.newLinePP, -1, -1));
        output.add (new ParseTreeNode (Constants.NODE_PP, indent, -1, -1));
      }
    }
    return output;
  }

  public String prettyPrintFinal (ParseTree tree, int step, boolean align, boolean lowercase, boolean html, boolean links) {
    int i, j;
    Vector output;
    ParseTreeNode prev, current, next;
    String result="", tempS, tempS1, indentLabel="", reducedIndentLabel;
    char tempC;

    output=prettyPrint (tree, 0, step, align, Constants.NONE, 0);
    if (tree.maxLabel>0)
      for (i=0; i<tree.maxLabel+1; i++)
        indentLabel+=Auxiliary.spacePP;
    result+=indentLabel;
    for (i=0; i<output.size (); i++) {
      current=(ParseTreeNode)(output.elementAt (i));
      if (i<output.size ()-1)
        next=(ParseTreeNode)(output.elementAt (i+1));
      else
        next=null;
      if ((current.symbolNum==Constants.NODE_PP) && (((String)(current.name)).startsWith (" ")) && (next!=null) && (next.symbolNum==Constants.NODE_NUM_LABEL)) {
        output.insertElementAt (current, i+3);
        output.removeElementAt (i);
        current=(ParseTreeNode)(output.elementAt (i));
        reducedIndentLabel="";
        for (j=0; j<tree.maxLabel-current.name.toString ().length (); j++)
          reducedIndentLabel+=Auxiliary.spacePP;
        result+=reducedIndentLabel;
        if (i<output.size ()-1)
          next=(ParseTreeNode)(output.elementAt (i+1));
        else
          next=null;
      }
      if (i>0)
        prev=(ParseTreeNode)(output.elementAt (i-1));
      else
        prev=null;
      if (html) {
        if ((links) && (current.hLinkFrom.length ()>0))
          result+="<a name=\""+current.hLinkFrom+"\">";
        if ((links) && (current.hLinkTo.length ()>0))
          result+="<a href=\"#"+current.hLinkTo+"\">";
        result+="<font color=\"#"+Classify.colour (current)+"\">";
        tempS="";
        tempS1=current.name.toString ();
        for (j=0; j<tempS1.length (); j++) {
          tempC=tempS1.charAt (j);
          if (tempC=='<')
            tempS+="&lt;";
          else
            if (tempC=='>')
              tempS+="&gt;";
            else
              if (tempC=='&')
                tempS+="&amp;";
              else
                if (tempC=='\"')
                  tempS+="&quot;";
                else
                  tempS+=tempC;
        }
      } else
        tempS=current.name.toString ();
      if ((lowercase) && (current.symbolNum!=sym.NIZ) && (current.symbolNum!=Constants.NODE_COMMENT))
        tempS=tempS.toLowerCase ();
      result+=tempS;
      if ((current.symbolNum==Constants.NODE_PP) && (((String)(current.name)).compareTo (Auxiliary.newLinePP)==0) && (i<output.size ()-2) && (((ParseTreeNode)(output.elementAt (i+2))).symbolNum!=Constants.NODE_NUM_LABEL))
        result+=indentLabel;
      if (html) {
        result+="</font>";
        if ((links) && (current.hLinkTo.length ()>0))
          result+="</a>";
        if ((links) && (current.hLinkFrom.length ()>0))
          result+="</a>";
      }
      if (((!Classify.noSpaceLR (current)) && (!Classify.noSpaceLR (next)) && (!Classify.noSpaceL (next)) && (!Classify.noSpaceR (current))) || (Classify.alwaysSpaceR (current)))
        result+=Auxiliary.spacePP;
    }
    return result;
  }

  public int spaces (ParseTree tree, int step, boolean align) {
    int i, max=0, tempI;

    if ((((String)name).compareTo ("def_konstante")==0) || (((String)name).compareTo ("def_tipa")==0) || (((String)name).compareTo ("def_spremenljivke")==0) || (((String)name).compareTo ("zapis")==0) || (((String)name).compareTo ("element_st_case")==0) || (((String)name).compareTo ("element_dela_s_case")==0))
      return ((String)(((ParseTreeNode)(children.elementAt (0))).prettyPrintFinal (tree, step, align, false, false, false))).length ();
    else
      if (children!=null)
        for (i=0; i<children.size (); i++) {
          tempI=((ParseTreeNode)(children.elementAt (i))).spaces (tree, step, align);
          if (tempI>max)
            max=tempI;
        }
    return max;
  }

  public void hLinks (ParseTree tree, int mode, Vector scopes) {
    int i, newMode=mode;
    String tempS;

    if (terminal) {
      if ((mode==Constants.HLINKS_TO_HERE) && (symbolNum==sym.IME))
        tree.addHLink (scopes, this);
      if ((mode==Constants.NONE) && (symbolNum==sym.IME))
        hLinkTo=tree.lookupHLink (scopes, (String)name);
    } else {
      if (((String)name).compareTo ("blok")==0)
        scopes.add (new Vector ());
      if (((String)name).compareTo ("vec_dodatkov_k_spremenljivki")==0)
        newMode=Constants.HLINKS_NOT_FROM_HERE;
    }
    if (children!=null)
      for (i=0; i<children.size (); i++) {
        if ((((String)name).compareTo ("def_konstante")==0) || (((String)name).compareTo ("def_tipa")==0) || (((String)name).compareTo ("def_spremenljivke")==0))
          if (i==0)
            newMode=Constants.HLINKS_TO_HERE;
          else
            newMode=Constants.NONE;
        if (((String)name).compareTo ("sp_po_vrednosti")==0)
          if (i==0)
            newMode=Constants.HLINKS_TO_HERE;
          else
            newMode=Constants.NONE;
        if ((((String)name).compareTo ("sp_po_referenci")==0) || (((String)name).compareTo ("sp_procedura")==0) || (((String)name).compareTo ("sp_funkcija")==0))
          if (i==1)
            newMode=Constants.HLINKS_TO_HERE;
          else
            newMode=Constants.NONE;
        if ((((String)name).compareTo ("def_procedure")==0) || (((String)name).compareTo ("def_funkcije")==0))
          if (i==1)
            newMode=Constants.HLINKS_TO_HERE;
          else
            newMode=Constants.NONE;
        if (((String)name).compareTo ("zapis")==0)
          if (i==0)
            newMode=Constants.HLINKS_NOT_FROM_HERE;
          else
            newMode=Constants.NONE;
        ((ParseTreeNode)(children.elementAt (i))).hLinks (tree, newMode, scopes);
      }
    if ((!terminal) && (((String)name).compareTo ("blok")==0))
      scopes.removeElementAt (scopes.size ()-1);
  }
}